var fs = require("fs"), path = require("path");
var async = require("async");
var ncp = require("ncp").ncp;
var $f = require("fluid");

var reportAssetsPath = path.resolve(__dirname, "../reporting/assets/");
var ideAssetsPath = path.resolve(__dirname, "../test_suite/client/");

module.exports = {
	reporting : {
		js : {
			main : mergeFiles([
				path.join(reportAssetsPath, "js/lib/jquery-1.7.2.min.js"),
				path.join(reportAssetsPath, "js/lib/jquery-ui-1.8.22.custom.min.js"),
				path.join(reportAssetsPath, "js/lib/jquery.cookie.js"),
				path.join(reportAssetsPath, "js/lib/jquery.ba-hashchange.min.js"),
				path.join(reportAssetsPath, "js/lib/jquery.ba-bbq.min.js"),
				path.join(reportAssetsPath, "js/report.js"),
				path.join(reportAssetsPath, "js/details.js"),
				path.join(reportAssetsPath, "js/stepDefinitions.js"),
				path.join(reportAssetsPath, "js/glossary.js"),
				path.join(reportAssetsPath, "js/main.js")
			]),
			jsTree : mergeFiles([
				path.join(reportAssetsPath, "js/lib/jstree/jquery.jstree.js")
			]),
			static : mergeFiles([
				path.join(reportAssetsPath, "js/static.js")
			])
		},
		css : {
			main : mergeFiles([
				path.join(reportAssetsPath, "css/style.css"),
				path.join(reportAssetsPath, "css/glossary.css"),
				path.join(reportAssetsPath, "css/detail.css"),
				path.join(reportAssetsPath, "css/summary.css"),
				path.join(reportAssetsPath, "css/report.css"),
				path.join(reportAssetsPath, "css/steps.css")
			]),
			jqueryUi : mergeFiles([
				path.join(reportAssetsPath, "css/theme/jquery-ui-1.8.22.custom.css")
			])
		}
	},
	testSuite : {
		js : {
			ide : mergeFiles([
				path.join(ideAssetsPath, "js/glossary.js"),
				path.join(ideAssetsPath, "js/testHistory.js"),
				path.join(ideAssetsPath, "js/ide.js")
			])
		},
		css : {
			main : mergeFiles([
				path.join(reportAssetsPath, "css/test-history.css")
			])
		}
	},
	buildAssetsDirectory : buildAssetsDirectory
};


function buildAssetsDirectory(options, callback) {
	$f(fs)
		.custom(function(callback) { fs.exists(options.output, function(exists) {
			if (!exists) { fs.mkdir(options.output, callback) } else { callback(); }
		});})
		.custom(function(callback) { fs.exists(path.join(options.output, "js"), function(exists) {
			if (!exists) { fs.mkdir(path.join(options.output, "js"), callback) } else { callback(); }
		});})
		.custom(function(callback) { fs.exists(path.join(options.output, "css"), function(exists) {
			if (!exists) { fs.mkdir(path.join(options.output, "css"), callback) } else { callback(); }
		});})
		.custom(function(callback) { fs.exists(path.join(options.output, "css/jqueryui"), function(exists) {
			if (!exists) { fs.mkdir(path.join(options.output, "css/jqueryui"), callback) } else { callback(); }
		});})
		.custom(function(callback) { fs.exists(path.join(options.output, "css/jqueryui/images"), function(exists) {
			if (!exists) { fs.mkdir(path.join(options.output, "css/jqueryui/images"), callback) } else { callback(); }
		});})
	.with(ncp)
		.self(path.join(reportAssetsPath, "images"), path.join(options.output, "images"), {})
		.self(path.join(reportAssetsPath, "js/lib/jstree"), path.join(options.output, "js/jstree"), {})
		.self(path.join(reportAssetsPath, "css/theme/images"), path.join(options.output, "css/jqueryui/images"), {})
	.with(module.exports.reporting.js)
		.main().static()
	.with(module.exports.reporting.css)
		.main().jqueryUi()
	.with(fs)
		.writeFile(path.join(options.output, "js/main.js"), $f("this.main[0]"))
		.writeFile(path.join(options.output, "js/static.js"), $f("this.static[0]"))
		.writeFile(path.join(options.output, "css/style.css"), $f("this.main[1]"))
		.writeFile(path.join(options.output, "css/jqueryui/style.css"), $f("this.jqueryUi[0]"))
	.go(function(err, result) {
		if (err) {callback(err);} else {
			callback();
		}
	});
}

function mergeFiles(files) {
	return function(callback) {
		var output = "";
		async.forEachSeries(files, function(file, callback) {
			fs.lstat(file, function(err, stat) {
				if (err) {
					callback({message : "Error reading stats for file: "+ file, error: err});
				} else {
					fs.readFile(file, function(err, contents) {
						if (err) { 
							callback({message : "Error reading file: "+ file, error: err});
						} else {
							output += contents.toString() +"\n";
							callback();
						}
					});
				}
			});
		}, function(err) {
			if (err) {callback(err);} else {
				callback(null, output);
			}
		});
	};
}