var fs = require("fs"), path = require("path"), ncp = require("ncp").ncp;
var handlebars = require("handlebars");
var ce = require("cloneextend");
var $f = require("fluid");
var templates = require("./templates/templates.js");
var handlebarsHelpers = require("./handlebarsHelpers.js");
var clientAssets = require("../lib/clientAssets.js");
	
	
exports.generate = function(options, callback) {		
	$f(ce.cloneextend(templates, options.templates))
		.report(handlebars).scenario(handlebars).group(handlebars)
		.summary(handlebars).glossary(handlebars).steps(handlebars)
		.justScenarios(handlebars)
	.with(fs)({name: "results"}).readFile(options.input)
	.with(require("./structureGenerator.js"))({name : "structureJson"}).generate(options)
	.with(require("./glossaryGenerator.js"))({name : "glossaryReport"}).generate(options)
	.with(require("./stepsGenerator.js"))({name : "stepsReport"}).generate(options)
	.go(function(err, res) {
		if (err) { callback(err); } else {
			handlebarsHelpers.register(handlebars);
		
			var resultsJson = JSON.parse(res.results[0].toString());
			var json = {
				structure : res.structureJson[0],
				summary : resultsJson.summary,
				glossary : res.glossaryReport[0].glossaryJson,
				stepFiles : res.stepsReport[0].stepsJson
			};
												
			var report = res.report[0](json);
																
			if (options.output) {
				buildAndSaveReport(options.output, report, json.glossary, callback);
			} else {
				callback(null, {
					report : report,
					scenarios : res.justScenarios[0](json.structure),
					summary : res.summary[0](json),
					steps : res.stepsReport[0].stepsHtml
				});
			}
		}
	});
};

function buildAndSaveReport(outputPath, report, glossaryJson, callback) {
	var outputFile = path.join(outputPath, "report.htm");
	clientAssets.buildAssetsDirectory({output : outputPath}, function(err) {
		if (err) {callback(err);} else {
			fs.writeFile(outputFile, report, "ascii", function(err) {
				if (err) {callback(err);} else {
					var glossaryDefinitionScript = "var glossaryDef = "+ JSON.stringify(glossaryJson, null, 3) + ";";
					fs.writeFile(path.join(outputPath, "glossary_definitions.js"), glossaryDefinitionScript, callback);											
				}
			});
		}
	});
}