var fs = require("fs");
var path = require("path");
var express = require("express");
var $f = require("fluid");
var async = require("async");
var handlebars = require("handlebars");
var reportingTemplates = require("../../reporting/templates/templates.js");
var clientAssets = require("../../lib/clientAssets.js")
var cukeTree = require("../../lib/cukeTree.js");


module.exports = function(options) {
	var app = options.app;
	var config = options.config;
	
	app.use(express.bodyParser());

	app.get("/", showTestSuitePage);
	app.post("/glossary", addTermToGlossary);
	
	app.use("/images", express.static(path.join(__dirname, "../../reporting/assets/images")));
		
	app.get("/js/main.js", concatFolderHandler(clientAssets.reporting.js.main, "text/javascript"));
	app.get("/js/jstree/jquery.jstree.js", concatFolderHandler(clientAssets.reporting.js.jsTree, "text/javascript"));
	app.use("/js/jstree", express.static(path.join(__dirname, "../../reporting/assets/js/lib/jstree")));
	app.get("/js/ide.js", concatFolderHandler(clientAssets.testSuite.js.ide, "text/javascript"));
	
	app.get("/css/style.css", concatFolderHandler(clientAssets.reporting.css.main, "text/css"));
	app.get("/css/jqueryui/style.css", concatFolderHandler(clientAssets.reporting.css.jqueryUi, "text/css"));
	app.use("/css/jqueryui/images", express.static(path.join(__dirname, "../../reporting/assets/css/theme/images")));
	app.get("/css/ide.css", concatFolderHandler(clientAssets.testSuite.css.main, "text/css"));
		
	function showTestSuitePage(request, response) {
		var emptyTemplate = handlebars.compile("");
		reportingTemplates.report(function(err, pageTemplate) {
			response.end(pageTemplate({suite:true}));
		});
	}
	
	function addTermToGlossary(request, response) {
		fs.readFile(config.glossaryFile, function(err, content) {
			if (err) {
				response.statusCode = 500;
				response.end(JSON.stringify({message : "Error reading glossary file", error: err}, null, 3));
			} else {
				var glossary = null, error = null;
				try { glossary = JSON.parse(content); }
				catch(e) { error = e; }
				
				if (error !== null) {
					response.statusCode = 500;
					response.end(JSON.stringify({message : "Error parsing glossary file", error: error}, null, 3));
				} else {
					var newTerm = {
						type : request.body.newTermType,
						name : request.body.newTermName,
						description : request.body.newTermDescription
					};
					
					var existingTerm = null;
					for(var i=0; i<glossary.length; i++) {
						if (glossary[i].name.toLowerCase() == newTerm.name.toLowerCase()) {
							existingTerm = glossary[i];
							break;
						}
					}
					if (existingTerm !== null) {
						response.statusCode = 403;
						response.end(JSON.stringify({ message : "Term already exists : "+ existingTerm.name}, null, 3));
					} else {
						glossary.push(newTerm);
						fs.writeFile(config.glossaryFile, JSON.stringify(glossary, null, 3), function(err) {
							if (err) {
								response.statusCode = 500;
								response.end(JSON.stringify({message : "Error writing glossary file", error: err}, null, 3));
							} else {
								response.statusCode = 201;
								response.end(JSON.stringify({ message : "Term successfully added : "+ newTerm.name}, null, 3));
							}
						});
					}
				}
			}
		});
	}

	function concatFolderHandler(assetFunction, contentType) {
		return function(request, response) {
			assetFunction(function(err, contents) {
				if (err) {
					response.statusCode = 500;
					response.end(JSON.stringify(err, null, 3));
				} else {
					response.setHeader("content-type", contentType);
					response.end(contents);
				}
			});			
		};
	}
};